#!/usr/local/bin/perl

#
# cookie.pl
#

###############################################################################
#
# Encoding characters
#
###############################################################################

@CookieEncodeChars = ('\%', '\+', '\;', '\,', '\=', '\&', '\:\:', '\s');

%CookieEncodeChars = ('\%',   '%25',
                      '\+',   '%2B',
                      '\;',   '%3B',
                      '\,',   '%2C',
                      '\=',   '%3D',
                      '\&',   '%26',
                      '\:\:', '%3A%3A',
                      '\s',   '+');

@CookieDecodeChars = ('\+', '\%3A\%3A', '\%26', '\%3D', '\%2C', '\%3B', '\%2B', '\%25');

%CookieDecodeChars = ('\+',       ' ',
                      '\%3A\%3A', '::',
                      '\%26',     '&',
                      '\%3D',     '=',
                      '\%2C',     ',',
                      '\%3B',     ';',
                      '\%2B',     '+',
                      '\%25',     '%');

###############################################################################
#
# Cookie functions.  Do not modify unless you're sure you know what you are doing.
#
###############################################################################

sub SetCookies {
   &LogItem("--> SetCookies()");
    local(@cookies) = @_;
    local($cookie, $value, $char);

	$CookieSet = 1;

    while( ($cookie, $value) = @cookies ) {

        foreach $char (@CookieEncodeChars) {
            $cookie =~ s/$char/$CookieEncodeChars{$char}/g;
            $value =~ s/$char/$CookieEncodeChars{$char}/g;
        }

        print 'Set-Cookie: ' . $cookie . '=' . $value . ';';
        print "\n";

        shift(@cookies); 
        shift(@cookies);
    }
   &LogItem("<-- SetCookies()");
}

sub GetCookies {
   &LogItem("--> GetCookies()");
    local(@ReturnCookies) = @_;
    local($cookieFlag) = 0;
    local($cookie, $value);

    if ($ENV{'HTTP_COOKIE'}) {

        if ($ReturnCookies[0] ne '') {

            foreach (split(/; /,$ENV{'HTTP_COOKIE'})) {
                ($cookie, $value) = split(/=/);

                foreach $char (@CookieDecodeChars) {
                    $cookie =~ s/$char/$CookieDecodeChars{$char}/g;
                    $value =~ s/$char/$CookieDecodeChars{$char}/g;
                }

                foreach $ReturnCookie (@ReturnCookies) {

                    if ($ReturnCookie eq $cookie) {
                        $Cookies{$cookie} = $value;
                        $cookieFlag = "1";
                    }
                }
            }

        }

        else {
            foreach (split(/; /,$ENV{'HTTP_COOKIE'})) {
                ($cookie, $value) = split(/=/);

                foreach $char (@CookieDecodeChars) {
                    $cookie =~ s/$char/$CookieDecodeChars{$char}/g;
                    $value =~ s/$char/$CookieDecodeChars{$char}/g;
                }

                $Cookies{$cookie} = $value;
            }
            $cookieFlag = 1;
        }
    }
   &LogItem("<-- GetCookies()");
    return $cookieFlag;
}

sub SetCompressedCookies {
   &LogItem("--> SetCompressedCookies()");
    local($cookieName,@cookies) = @_;
    local($cookie, $value, $cookieValue);

    while ( ($cookie,$value) = @cookies ) {
        foreach $char (@CookieEncodeChars) {
            $cookie =~ s/$char/$CookieEncodeChars{$char}/g;
            $value =~ s/$char/$CookieEncodeChars{$char}/g;
        }

        if ($cookieValue) {
            $cookieValue .= '&' . $cookie . '::' . $value;
        }
        else {
            $cookieValue = $cookie . '::' . $value;
        }

        shift(@cookies); 
        shift(@cookies);
    }
    &SetCookies("$cookieName","$cookieValue");
   &LogItem("<-- SetCompressedCookies()");
}

sub GetCompressedCookies {
   &LogItem("--> GetCompressedCookies()");
    local($cookieName,@ReturnCookies) = @_;
    local($cookieFlag) = 0;
    local($ReturnCookie, $cookie, $value);

    if (&GetCookies($cookieName)) {
        if ($ReturnCookies[0] ne '') {
            foreach (split(/&/,$Cookies{$cookieName})) {
                ($cookie, $value) = split(/::/);

                foreach $char (@CookieDecodeChars) {
                    $cookie =~ s/$char/$CookieDecodeChars{$char}/g;
                    $value =~ s/$char/$CookieDecodeChars{$char}/g;
                }
                foreach $ReturnCookie (@ReturnCookies) {
                    if ($ReturnCookie eq $cookie) {
                        $Cookies{$cookie} = $value;
                        $cookieFlag = 1;
                    }
                }
            }
        }
        else {
            foreach (split(/&/,$Cookies{$cookieName})) {
                ($cookie, $value) = split(/::/);

                foreach $char (@CookieDecodeChars) {
                    $cookie =~ s/$char/$CookieDecodeChars{$char}/g;
                    $value =~ s/$char/$CookieDecodeChars{$char}/g;
                }

                $Cookies{$cookie} = $value;
            }
            $cookieFlag = 1;
        }
        delete($Cookies{$cookieName});
    }
   &LogItem("<-- GetCompressedCookies()");
    return $cookieFlag;
}

# Needed for reuire statement in Main

1;
