#!/usr/local/bin/perl

#
# sockmsg.pl
#

require 5.002;
use strict;
use Socket;
use Carp;

use vars qw($argCount @argList @bufMsg $bufSize $bufIdx);
use vars qw($ANY $INTEGER $BOOLEAN $STRING $FALSE $TRUE);

###############################################################################
#
# Constants
#
###############################################################################

$FALSE   = 0;
$TRUE    = 1;

$ANY     = 0;
$INTEGER = 1;
$BOOLEAN = 2;
$STRING  = 3;

###############################################################################
#
# Socket message functions. Do not modify unless you are sure you know what
# what you are doing!
#
###############################################################################

sub Initialize {
    &LogItem("--> Initialize()");
	$bufSize = 0;
	$argCount = 0;
	$bufIdx = 0;
	@bufMsg = ();
	@argList = ();
    &LogItem("<-- Initialize()");
}

sub CreateSocket {
    &LogItem("--> CreateSocket()");
	my($remote, $port) = @_;
	my($iaddr, $paddr, $proto, $try );

	# Initialize Variables
	&Initialize();

	if ($port =~/\D/) { 
		$port = getservbyname($port, 'tcp') 
	} die "No port" unless $port;

	$iaddr = inet_aton($remote);
	$paddr = sockaddr_in($port, $iaddr);

	$proto = getprotobyname('tcp');
	if(socket(SOCK, PF_INET, SOCK_STREAM, $proto)) {
		if(connect(SOCK, $paddr)) {
            &LogItem("<-- CreateSocket()");
			return($TRUE);
		}
        &LogItem("<-- CreateSocket():Failed ($!)");
	}
    &LogItem("<-- CreateSocket():Failed ($!)");
	return($FALSE);
}

sub CloseSocket {
    &LogItem("--> CloseSocket()");
	close(SOCK) or die "Close failed: $!";;
    &LogItem("<-- CloseSocket()");
}

################################################################
#
#	Send Message
#
################################################################

sub PackInteger {
    #&LogItem("--> PackInteger()");
	my($int) = @_;

    $bufMsg[$bufIdx++] = $INTEGER;
    $bufMsg[$bufIdx++] = ($int & 0xff000000) >> 24;
    $bufMsg[$bufIdx++] = ($int & 0x00ff0000) >> 16;
    $bufMsg[$bufIdx++] = ($int & 0x0000ff00) >> 8;
    $bufMsg[$bufIdx++] = $int & 0x000000ff;
    #&LogItem("<-- PackInteger()");
}

sub PackBoolean {
    #&LogItem("--> PackBoolean()");
	my($bool) = @_;

    $bufMsg[$bufIdx++] = $BOOLEAN;
    if($bool) {
        $bufMsg[$bufIdx++] = 1;
    }
    else {
        $bufMsg[$bufIdx++] = 0;
    }
    #&LogItem("<-- PackBoolean()");
}


sub PackString {
    #&LogItem("--> PackString()");
	my($len, $i, $s);
	my($str) = @_;

    $bufMsg[$bufIdx++] = $STRING;

    # Pack string length
    $len = length($str);
    $bufMsg[$bufIdx++] = ($len & 0xff000000) >> 24;
    $bufMsg[$bufIdx++] = ($len & 0x00ff0000) >> 16;
    $bufMsg[$bufIdx++] = ($len & 0x0000ff00) >> 8;
    $bufMsg[$bufIdx++] = $len & 0x000000ff;

    # Pack string characters
    for ( $i = 0; $i < $len; $i++ )
    {
        $s = substr($str, $i, 1);
		$s = vec($s, 0, 8);
        $bufMsg[$bufIdx++] = (($s & 0xff00) >> 8) & 0xff;
        $bufMsg[$bufIdx++] = $s & 0x00ff;
    }
    #&LogItem("<-- PackString()");
}

sub AddArg{
    #&LogItem("--> AddArg()");
	my($arg, $type) = @_;

	if($type == $INTEGER) {
		PackInteger($arg);
		$bufSize += 5;
	}
	elsif($type == $BOOLEAN) {
		PackBoolean($arg);
		$bufSize += 2;
	}
	elsif($type == $STRING) {
		PackString($arg);
		$bufSize += 5 + 2 * length($arg);
	}
	$argCount++;
    #&LogItem("<-- AddArg()");
}

sub SendMsg {
    &LogItem("--> SendMsg()");
	my($written, $offset, $sendBuf, @bufHdr); 
	my($cmd, $subCmd) = @_;

    # Calculate the size of the buffer.
    $bufSize += 12; # cmd, subCmd, argCount

	$bufIdx=0;
    # Pack buffer size, message type, subtype, and number of arguments.
    $bufHdr[$bufIdx++] = ($bufSize & 0xff000000) >> 24;
    $bufHdr[$bufIdx++] = ($bufSize & 0x00ff0000) >> 16;
    $bufHdr[$bufIdx++] = ($bufSize & 0x0000ff00) >> 8;
    $bufHdr[$bufIdx++] = ($bufSize & 0x000000ff);

    $bufHdr[$bufIdx++] = ($cmd & 0xff000000) >> 24;
    $bufHdr[$bufIdx++] = ($cmd & 0x00ff0000) >> 16;
    $bufHdr[$bufIdx++] = ($cmd & 0x0000ff00) >> 8;
    $bufHdr[$bufIdx++] = ($cmd & 0x000000ff);

    $bufHdr[$bufIdx++] = ($subCmd & 0xff000000) >> 24;
    $bufHdr[$bufIdx++] = ($subCmd & 0x00ff0000) >> 16;
    $bufHdr[$bufIdx++] = ($subCmd & 0x0000ff00) >> 8;
    $bufHdr[$bufIdx++] = ($subCmd & 0x000000ff);

    $bufHdr[$bufIdx++] = ($argCount & 0xff000000) >> 24;
    $bufHdr[$bufIdx++] = ($argCount & 0x00ff0000) >> 16;
    $bufHdr[$bufIdx++] = ($argCount & 0x0000ff00) >> 8;
    $bufHdr[$bufIdx++] = ($argCount & 0x000000ff);
    
	$bufSize += 4; # Add the 4 bytes for the size
	$sendBuf = pack "c$bufSize", @bufHdr, @bufMsg;

	$offset = 0;
	while($bufSize) {
		$written = syswrite(SOCK, $sendBuf, $bufSize, $offset);
		if(defined $written) {
			$bufSize -= $written;
			$offset += $written;
		}
		else {
	        # Check for end of stream.
			&Initialize();
            &LogItem("! Stream ended early.");
            &LogItem("<-- SendMsg()");
            return($FALSE);
		}
	};

	# Clear Variables
	&Initialize();
    &LogItem("<-- SendMsg()");
}



################################################################
#
#	Recieve Message
#
################################################################

sub UnpackArg {
    #&LogItem("--> UnpackArg()");
	my($arg);
	my($length) = @_;
    if( $length < 1) {
        #&LogItem("! Length is less than 1.");
        #&LogItem("<-- UnpackArg()");
        return;
    }
    
    if($bufMsg[$bufIdx] == $INTEGER) {
		$arg = UnpackInteger($length);
	}
	elsif($bufMsg[$bufIdx] == $BOOLEAN) {
		$arg = UnpackBool($length);
	}
	elsif($bufMsg[$bufIdx] == $STRING) {
		$arg = UnpackString($length);
	}
    #&LogItem("<-- UnpackArg()");
	return($arg);
}

sub UnpackInteger {
    #&LogItem("--> UnpackInteger()");
	my($int);
	my($length) = @_;

    if($bufMsg[$bufIdx++] != $INTEGER) {
        #&LogItem("! Not an Integer.");
        #&LogItem("<-- UnpackInteger()");
        return;
    }

    if ( $length < 5 ) {
        #&LogItem("! Length less than 5.");
        #&LogItem("<-- UnpackInteger()");
        return;
    }
    
    $int = 0;
    $int |= $bufMsg[$bufIdx++] << 24 & 0xff000000;
    $int |= $bufMsg[$bufIdx++] << 16 & 0x00ff0000;
    $int |= $bufMsg[$bufIdx++] << 8  & 0x0000ff00;
    $int |= $bufMsg[$bufIdx++] & 0x000000ff;

    #&LogItem("<-- UnpackInteger()");
    return($int);
}

sub UnpackBool {
    #&LogItem("--> UnpackBool()");
	my($bool);
	my($length) = @_;

    if($bufMsg[$bufIdx++] != $BOOLEAN) {
        #&LogItem("! Not a Boolean.");
        #&LogItem("<-- UnpackBool()");
        return;
    }

    if($length < 2) {
        #&LogItem("! Length less than 2.");
        #&LogItem("<-- UnpackBool()");
        return;
    }

    $bool = $bufMsg[$bufIdx++];

    #&LogItem("<-- UnpackBool()");
    return($bool);
}

sub UnpackString {
    #&LogItem("--> UnpackString()");
	my($len, $i, $s, $str, $tmp);
	my($length) = @_;

    if($bufMsg[$bufIdx++] != $STRING) {
        #&LogItem("! Not a String.");
        #&LogItem("<-- UnpackString()");
        return;
    }

    if($length < 5) {
        #&LogItem("! Length less than 5.");
        #&LogItem("<-- UnpackString()");
        return;
    }
    
    # Unpack string length
    $len = 0;
    $len |= $bufMsg[$bufIdx++] << 24 & 0xff000000;
    $len |= $bufMsg[$bufIdx++] << 16 & 0x00ff0000;
    $len |= $bufMsg[$bufIdx++] << 8  & 0x0000ff00;
    $len |= $bufMsg[$bufIdx++] & 0x000000ff;

    if($len < 0) {
        #&LogItem("! Invalid Format.");
        #&LogItem("<-- UnpackString()");
        return;
    }
    
    if($length < 5 + 2 * $len) {
        #&LogItem("! Invalid Format.");
        #&LogItem("<-- UnpackString()");
        return;
    }

    # Unpack string characters
	$str = "";
    for($i=0; $i < $len; $i++) {
        $s = 0;
        $s |= ( $bufMsg[$bufIdx++] << 8 ) & 0xff00;
        $s |= $bufMsg[$bufIdx++] & 0x00ff;
		$tmp = chr($s);
		$str = "$str$tmp";
    }

    #&LogItem("<-- UnpackString()");
    return($str);
}

sub RecvMsg {
    &LogItem("--> RecvMsg()");
    my($expect, $count, $ret, $arg, $i, @retList, $cmd, $subCmd, $nArgs, $bufLen, @bufLen, $tmp, $msg);
    
	# Initialize Variables
	&Initialize();
	@retList = ();

    $expect = 4;
    $count  = 0;
    $bufIdx = 0;

	$bufLen = "";
    while($count < $expect) {
        $ret = sysread(SOCK, $tmp, $expect - $count);
        # Check for end of stream.
        if($ret <= 0) {
            &LogItem("! Stream ended early.");
            &LogItem("<-- RecvMsg()");
            return($FALSE);
        }
		$bufLen = "$bufLen$tmp";
        $count  += $ret;
    }
	@bufLen = unpack("c4", $bufLen);
    # Make sure buffer is big enough.
    $bufIdx = 0;
    $expect = 0;
    $expect |= $bufLen[$bufIdx++] << 24 & 0xff000000;
    $expect |= $bufLen[$bufIdx++] << 16 & 0x00ff0000;
    $expect |= $bufLen[$bufIdx++] << 8  & 0x0000ff00;
    $expect |= $bufLen[$bufIdx++] & 0x000000ff;

    # Read the message.
    $count  = 0;
    $bufIdx = 0;

	$msg = "";
    while($count < $expect) {
        $ret = sysread(SOCK, $tmp, $expect - $count);

        # Check for end of stream.
        if($ret <= 0) {
            &LogItem("! Stream ended early.");
            &LogItem("<-- RecvMsg()");
            return($FALSE);
        }

		$msg = "$msg$tmp";
        $count  += $ret;
    }

    # Decode the message and dispatch it.
	@bufMsg = unpack("c$expect", $msg);
    $bufIdx = 0;

    $cmd = 0;
    $cmd |= $bufMsg[$bufIdx++] << 24 & 0xff000000;
    $cmd |= $bufMsg[$bufIdx++] << 16 & 0x00ff0000;
    $cmd |= $bufMsg[$bufIdx++] << 8  & 0x0000ff00;
    $cmd |= $bufMsg[$bufIdx++] & 0x000000ff;
	$retList[0] = $cmd;

    $subCmd = 0;
    $subCmd |= $bufMsg[$bufIdx++] << 24 & 0xff000000;
    $subCmd |= $bufMsg[$bufIdx++] << 16 & 0x00ff0000;
    $subCmd |= $bufMsg[$bufIdx++] << 8  & 0x0000ff00;
    $subCmd |= $bufMsg[$bufIdx++] & 0x000000ff;
	$retList[1] = $subCmd;

    $nArgs = 0;
    $nArgs |= $bufMsg[$bufIdx++] << 24 & 0xff000000;
    $nArgs |= $bufMsg[$bufIdx++] << 16 & 0x00ff0000;
    $nArgs |= $bufMsg[$bufIdx++] << 8  & 0x0000ff00;
    $nArgs |= $bufMsg[$bufIdx++] & 0x000000ff;

	$argCount = 0;
    for($i=0; $i < $nArgs; $i++) {
        $arg = &UnpackArg($count - $bufIdx + 4 );
        $argList[$argCount++] = $arg;
    }

	# The Last Arg is always the error code
	$retList[2] = $argList[$argCount-1];
    &LogItem("<-- RecvMsg()");
	return(@retList);
}

sub SetArgs
{
    #&LogItem("--> SetArgs()");
    my ($hashRef) = @_;
    my $i = 0;
    my $metadata;

    foreach $metadata (keys %$hashRef) {
        $i++;
    }
    &AddArg( $i, $INTEGER );

    foreach $metadata (keys %$hashRef) {
        &AddArg( $metadata, $STRING );
        &AddArg( $$hashRef{$metadata}, $STRING );
    }
    #&LogItem("<-- SetArgs()");
}

sub GetArgs
{
    #&LogItem("--> GetArgs()");
    my ($hashRef) = @_;
    my $i;
    my $metadata;

    $i = 0;
    while ($i<$argCount) {
        $metadata = $argList[$i++];
        $$hashRef{$metadata} = $argList[$i++];
    }
    #&LogItem("<-- GetArgs()");
}

sub Sleep
{
    my ($waitTime) = @_;
    my ($timer, $endTime);

	$timer = time();
	$endTime = $timer + $waitTime;
    while($timer < $endTime) {
		$timer = time();
	}
}

# Needed for reuire statement in Main

1;
