#!/usr/local/bin/perl

#
# submit.pl
#
# DisplaySubmit()
# DisplayUpdate()
# Submit()
# Update()
#

###############################################################################
# Procedure:   DisplaySubmit
#
# FUNC:        DISP_SUBMIT
#
# Description: Displays the 'Submit Problem' screen.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplaySubmit {
    &LogItem("--> DisplaySubmit()");
	&ValidateUser();

    if (&GetSystemFlag()) {
        &SystemBox();
    }
    &DescriptionBox();

    if ($userID eq '') {
        # Use the email address field for anonymous users.
        $VAR{'EMAIL_BOX_VAR'} = $_EMAIL_BOX_TAG;
        $VAR{'EMAIL_ADDRESS_VAR'} = $FORM{'EMAIL_ADDRESS'};
    }
	&SetAutoLogoutTag();
    &set_submit_strings();
    &EchoFile($HTML_PATH, "submit.htm");
    &LogItem("<-- DisplaySubmit()");
}

###############################################################################
# Procedure:   Submit
#
# FUNC:        SUBMIT
#
# Description: Submits a new problem to the database.  Opens the problem
#              for viewing on success.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub Submit {
    &LogItem("--> Submit()");
	&ValidateUser();

	if ((!$FORM{'DESCRIPTION'}) || (($userID eq '')&&(!$FORM{'EMAIL_ADDRESS'}))) {
        $list = '';
        if (!$FORM{'DESCRIPTION'}) {
            $list = &Bold( $DESCRIPTION ) . ', ';
        }
        if (($userID eq '')&&(!$FORM{'EMAIL_ADDRESS'})) {
            $list .= &Bold( $EMAIL_ADDRESS ) . ', ';
        }
        chop $list; chop $list;  # Remove trailing ', '
        $VAR{'INFORMATION_VAR'} = &InfoStyle( "$MANDATORY_FIELDS $list", "RED" );
        &DisplaySubmit();
        &LogItem("<-- Submit()");
        exit;
    }

    $system = $FORM{'SYSTEM'};
    $description = $FORM{'DESCRIPTION'};
    $email = $FORM{'EMAIL_ADDRESS'};
    if ($userID eq '') {
        $url = "http://$ENV{'HTTP_HOST'}$SCRIPT_PATH/web_access.cgi?FUNC=ANONYMOUS_LOGIN&PROBLEM_ID=";
    }
    else {
        $url = "http://$ENV{'HTTP_HOST'}$SCRIPT_PATH/web_access.cgi?FUNC=DISP_LOGIN";
    }

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_CreateProblem', $STRING);
		&AddArg($userAuthID, $STRING);
		&AddArg($system, $STRING);
		&AddArg($description, $STRING);
        &AddArg($email, $STRING);
        &AddArg($url, $STRING);
		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
			if($argList[0] == $WA_SUCCESS) {
                # Success.  Display the details of the problem.
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $PROBLEM_SUBMITTED_SUCCESSFULLY, "BLUE" );
                $FORM{'PROBLEM_ID'} = $argList[1];
                &ProblemDetail();
			}
			else {
                &AppServerError( 'Submit', $argList[0] );
			}
			$done = 1;
		}
		else {
			$try++;
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
        &DaemonError( 'Submit', $errCode );
	}
    &LogItem("<-- Submit()");
}

###############################################################################
# Procedure:   DisplayUpdate
#
# FUNC:        DISP_UPDATE
#
# Description: Dipslays the problem update screen.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayUpdate {
    &LogItem("--> DisplayUpdate()");
    &ValidateUser();

	if ($FORM{'PROBLEM_ID'}) {
		$VAR{'PROBLEM_ID_VAR'} = $FORM{'PROBLEM_ID'};
		&SetAutoLogoutTag();
        &set_update_strings();
		&EchoFile($HTML_PATH, "update.htm");
	}
	else {
        &LogItem( "! DisplayUpdate: Problem ID unknown" );
		&Error(); # Unexpected Error
	}
    &LogItem("<-- DisplayUpdate()");
}

###############################################################################
# Procedure:   Update
#
# FUNC:        UPDATE
#
# Description: Submits an update request to the database.  Returns to the problem
#              detail screen on success.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub Update {
    &LogItem("--> Update()");
    &ValidateUser();

	if ($FORM{'DESCRIPTION'}) {
        if (!$FORM{'PROBLEM_ID'}) {
            &LogItem( "! Update: Problem ID unknown" );
            &Error(); # Unexpected Error
        }

		$problemID = $FORM{'PROBLEM_ID'};
		$description = $FORM{'DESCRIPTION'};

	    if ($userAuthID eq 0) {
	        $url = "http://$ENV{'HTTP_HOST'}$SCRIPT_PATH/web_access.cgi?FUNC=ANONYMOUS_LOGIN&PROBLEM_ID=";
	    }
	    else {
	        $url = "http://$ENV{'HTTP_HOST'}$SCRIPT_PATH/web_access.cgi?FUNC=DISP_LOGIN";
	    }

		$try = 0;
		$done = 0;
		while(($try < $MAX_RETRIES)&&($done eq 0)) {
		    if(!&CreateSocket($DAEMON, $PORT)) {
				next;
			}
            &AddArg($KML_FILE, $STRING);
            &AddArg('WA_UpdateProblem', $STRING);
			&AddArg($userAuthID, $STRING);
			&AddArg($problemID, $STRING);
			&AddArg($description, $STRING);
			&AddArg($url, $STRING);
            &SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_CMD);
			($cmd, $subCmd, $errCode) = &RecvMsg();
			&CloseSocket();
			if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
				if($argList[0] == $WA_SUCCESS) {
					$VAR{'INFORMATION_VAR'} = &InfoStyle( $REQUEST_UPDATED, "BLUE" );
					&ProblemDetail();
				}
				else { 
                    &AppServerError( 'Update', $argList[0] );
				}
				$done = 1;
			}
			else {
				$try++;
			}
	    } continue {
			$try++;
		}

		if($done eq 0) {
			&DaemonError( 'Update', $errCode );
		}
	}
	else { # Description field is required.
        $list = &Bold( $DESCRIPTION );
        $VAR{'INFORMATION_VAR'} = &InfoStyle( "$MANDATORY_FIELDS $list", "RED" );
        &DisplayUpdate();
	}
    &LogItem("<-- Update()");
}

# Needed for reuire statement in Main

1;
