#!/usr/local/bin/perl

#
# view.pl
#
# DisplayProblemInquiry()
# GetProblemHistory()
# GetWorkHistory()
# ProblemDetail()
# ProblemInquiry()
#

###############################################################################
# Procedure:   DisplayProblemInquiry
#
# FUNC:        DISP_PROB_INQUIRY
#
# Description: Displays the problem inquiry screen.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub DisplayProblemInquiry {
     &LogItem("--> DisplayProblemInquiry()");
	&ValidateUser();

	if((&GetSystemList())&&(&GetProbCodeList())) {
        $VAR{'PROBLEM_ID_VAR'} = $FORM{'PROBLEM_ID'};
        $VAR{'BEGIN_DATE_VAR'} = $FORM{'BEGIN_DATE'};
        $VAR{'END_DATE_VAR'} = $FORM{'END_DATE'};
        $VAR{'DATE_FORMAT_VAR'} = $DATE_INPUT_FORMAT;

        if (($viewRight == 2) && ($organizationID ne '')) {
            # Add this checkbox iff the user has rights to view
            # problems in his/her organization.
            $VAR{'INCLUDE_ORG_CHECKBOX_VAR'} = $_INCLUDE_ORG_CHECKBOX_TAG;
            if ($FORM{'INCLUDE_ORG'}) {
                $VAR{'INCLUDE_ORG_VAR'} = 'CHECKED';
            }
        }
		elsif ($viewRight == 3) {
            # Add this checkbox iff the user has rights to view
            # all problems
            $VAR{'INCLUDE_ORG_CHECKBOX_VAR'} = $_INCLUDE_ALL_CHECKBOX_TAG;
            if ($FORM{'INCLUDE_ORG'}) {
                $VAR{'INCLUDE_ORG_VAR'} = 'CHECKED';
            }
        }
		&SetAutoLogoutTag();
        &set_prob_inquiry_strings();
		&EchoFile($HTML_PATH, "prob_inquiry.htm");
	}
     &LogItem("<-- DisplayProblemInquiry()");
}

###############################################################################
# Procedure:   ProblemInquiry
#
# FUNC:        PROB_INQUIRY
#
# Description: Sends inquiry information to the daemon and displays a list
#              of problems matched from the search criteria.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub ProblemInquiry {
     &LogItem("--> ProblemInquiry()");
	&ValidateUser();

    $begin_date = &FormatDaemonDateStr($FORM{'BEGIN_DATE'});
    if (($FORM{'BEGIN_DATE'} ne '') && ($begin_date eq '')) {
        $VAR{'INFORMATION_VAR'} = &InfoStyle( $DATE_FORMAT_ERROR, "RED" );
        &DisplayProblemInquiry();
        &LogItem("<-- ProblemInquiry()");
        exit;
    }

    $end_date   = &FormatDaemonDateStr($FORM{'END_DATE'});
    if (($FORM{'END_DATE'} ne '') && ($end_date eq '')) {
        $VAR{'INFORMATION_VAR'} = &InfoStyle( $DATE_FORMAT_ERROR, "RED" );
        &DisplayProblemInquiry();
        &LogItem("<-- ProblemInquiry()");
        exit;
    }

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE, $STRING);
        &AddArg('WA_ProblemInquiry', $STRING);
		&AddArg($userAuthID, $STRING);
        if (($FORM{'INCLUDE_ORG'} eq 'on')&&($viewRight >= 2)) {
                                                # View flag restricts search
            &AddArg("$viewRight", $STRING);     # 2 = all problems from user's
        }                                       #     default org
        else {
            &AddArg('1', $STRING);              # 1 = send their right
        }
		&AddArg($FORM{'PROBLEM_ID'}, $STRING);
		&AddArg($FORM{'SYSTEM'}, $STRING);
		&AddArg($FORM{'PROBLEM_CODE'}, $STRING);
		&AddArg($begin_date, $STRING);
		&AddArg($end_date, $STRING);

		if( exists($FORM{'START_IDX'})) {
			$startIdx = $FORM{'START_IDX'};
		}
		else {
			$startIdx = 1;
		}
        &AddArg($startIdx, $STRING);
        &AddArg($MAX_PROB, $STRING);

		&SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
			$argIdx = 1;
			$probList = "";
			if($argList[0] > 0) {
				for($i=0; $i < $argList[0]; $i++) {
					$probList = "$probList<TR>\n";
					$probList = "$probList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"30%\"><A HREF=\"$SCRIPT_PATH/web_access.cgi?FUNC=PROB_DETAIL&PROBLEM_ID=$argList[$argIdx]\">$argList[$argIdx++]</A></TD>\n"; # Problem ID
					$probList = "$probList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"20%\">$argList[$argIdx++]</TD>\n"; # Status
					$probList = "$probList<TD ALIGN=LEFT VALIGN=TOP WIDTH=\"50%\">$argList[$argIdx++]</TD>\n"; # Description
					$probList = "$probList</TR>\n";
				}
                $VAR{'PROBLEM_LIST_VAR'} = $probList;

                $totalMatches = $argList[$argIdx];
                if ($startIdx > 1) {
                    $prev = 1;
                }
                else {
                    $prev = 0;
                }
                if (($startIdx + $MAX_PROB - 1) < $totalMatches) {
                    $next = 1;
                }
                else {
                    $next = 0;
                }
                if(exists($FORM{'NEXT'})) {
					$VAR{'PAGE_NUMBER_VAR'} = $FORM{'PAGE_NUMBER'} + 1;
				}
				elsif(exists($FORM{'PREV'})) {
					$VAR{'PAGE_NUMBER_VAR'} = $FORM{'PAGE_NUMBER'} - 1;
				}
				else {
					$VAR{'PAGE_NUMBER_VAR'} = 1;
				}

                # Set variables for number of problems matched.
                $VAR{'TOTAL_MATCHES_VAR'} = $totalMatches;
                $tmp = $totalMatches / $MAX_PROB;  # NOTE: floating-point division.

                # Set pages equal to $tmp rounded up to the next integer.
                $VAR{'TOTAL_PAGES_VAR'} = ($tmp > (int $tmp)) ? (int $tmp) + 1 : (int $tmp);
				$VAR{'PREV_NEXT_BUTTONS_VAR'} = &PrevNextButtons($startIdx, $prev, $next);
				&SetAutoLogoutTag();
                &set_prob_list_strings();
				&EchoFile($HTML_PATH, "prob_list.htm");
			}
			elsif ($argList[0] == 0) {
                # No problems matched.  Inform the user, and repeat the search criteria.
                $VAR{'INFORMATION_VAR'} = &InfoStyle( $NO_REQUEST_RET, "RED" );
                &DisplayProblemInquiry();
			}
            else {
                &AppServerError( 'ProblemInquiry', $argList[0] );                
            }
			$done = 1;
		}
		else {
			$try++;
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
		&DaemonError( 'ProblemInquiry', $errCode );
	}
    &LogItem("<-- ProblemInquiry()");
}

###############################################################################
# Procedure:   ProblemDetail
#
# FUNC:        PROB_DETAIL
#
# Description: Displays detailed information about a problem.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub ProblemDetail {
    &LogItem("--> ProblemDetail()");
	&ValidateUser();

	$problemID = $FORM{'PROBLEM_ID'};

	$try = 0;
	$done = 0;
	while(($try < $MAX_RETRIES)&&($done eq 0)) {
	    if(!&CreateSocket($DAEMON, $PORT)) {
			next;
		}
        &AddArg($KML_FILE , $STRING);
        &AddArg('WA_ProblemDetail', $STRING);
		&AddArg($problemID, $STRING);
        &SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
		($cmd, $subCmd, $errCode) = &RecvMsg();
		&CloseSocket();
		if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
			if($argList[0] == $WA_SUCCESS) {
				$argIdx = 1;
				$VAR{'PROBLEM_ID_VAR'}   = $argList[$argIdx++];
				$VAR{'PROBLEM_CODE_VAR'} = $argList[$argIdx++];
				$VAR{'CONTACT_VAR'} = $argList[$argIdx++];
				$VAR{'ORGANIZATION_VAR'} = $argList[$argIdx++];
				$problemOrganizationID = $argList[$argIdx++];

                # Check rights.  Bypass right checking if the user just submitted a problem.
                &RightViolation($VIEW_RIGHT) if (($FORM{'FUNC'} ne 'SUBMIT')&&(!&VerifyViewRight()));

				$dateTime = &FormatDateTimeStr($argList[$argIdx++], $argList[$argIdx++]);
				$VAR{'DATE_TIME_SUBMITTED_VAR'} = $dateTime;
				$dateTime = &FormatDateTimeStr($argList[$argIdx++], $argList[$argIdx++]);
				$VAR{'DATE_TIME_CLOSED_VAR'} = $dateTime;
				$VAR{'SYSTEM_VAR'} = $argList[$argIdx++];
				$VAR{'DESCRIPTION_VAR'} = &FormatString( $argList[$argIdx++] );
				$VAR{'RESULT_VAR'} = &FormatString( $argList[$argIdx++] );

                # Display update and view buttons depending on the rights.
                if (&VerifyUpdateRight()) {
                    $VAR{'_PROB_DETAIL_INSTRUCTIONS_1_VAR'} = $_PROB_DETAIL_INSTRUCTIONS_1;
                    $VAR{'UPDATE_PROBLEM_VAR'} = $_UPDATE_PROBLEM_BUTTON_TAG;
                }
                # Anonymous users can view details about their problems!
                if (($viewRight >= 1) || ($a_viewRight >= 1)) {
                    $VAR{'_PROB_DETAIL_INSTRUCTIONS_2_VAR'} = $_PROB_DETAIL_INSTRUCTIONS_2;
                    $VAR{'VIEW_PROBLEM_HISTORY_VAR'} = $_VIEW_PROBLEM_HISTORY_BUTTON_TAG;
                    $VAR{'_PROB_DETAIL_INSTRUCTIONS_3_VAR'} = $_PROB_DETAIL_INSTRUCTIONS_3;
                    $VAR{'VIEW_WORK_HISTORY_VAR'} = $_VIEW_WORK_HISTORY_BUTTON_TAG;
                }
				&SetAutoLogoutTag();
                &set_prob_detail_strings();
				&EchoFile($HTML_PATH, "prob_detail.htm");
			}
			else {
                &AppServerError( 'ProblemDetail', $argList[0] );
			}
			$done = 1;
		}
		else {
			$try++;
		}
    } continue {
		$try++;
	}

	if($done eq 0) {
        &DaemonError( 'ProblemDetail', $errCode );
	}
    &LogItem("<-- ProblemDetail()");
}

###############################################################################
# Procedure:   GetProblemHistory
#
# FUNC:        VIEW_PROB_HISTORY
#
# Description: Displays problem history records for a given problem ID.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub GetProblemHistory {
    &LogItem("--> GetProblemHistory()");
	&ValidateUser();

	if ($FORM{'PROBLEM_ID'}) {
		$problemID = $FORM{'PROBLEM_ID'};

		$try = 0;
		$done = 0;
		while(($try < $MAX_RETRIES)&&($done eq 0)) {
		    if(!&CreateSocket($DAEMON, $PORT)) {
				next;
			}
            &AddArg($KML_FILE, $STRING);
            &AddArg('WA_GetProblemHistory', $STRING);
			&AddArg($problemID, $STRING);
            &SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
			($cmd, $subCmd, $errCode) = &RecvMsg();
			&CloseSocket();
			if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
				$argIdx = 1;
				$histList = "";
				if($argList[0] > 0) {
					for($i=0; $i < $argList[0]; $i++) {
						$histList = "$histList<TR>\n";
						$date = &FormatDateStr($argList[$argIdx++]);
                        $time = &FormatTimeStr($argList[$argIdx++]);
						$histList = "$histList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"15%\">$date</TD>\n"; 				# Date
                        $histList = "$histList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"15%\">$time</TD>\n";                # Time
                        $histList = "$histList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"15%\">$argList[$argIdx++]</TD>\n";  # Type
						$histList = "$histList<TD ALIGN=LEFT VALIGN=TOP WIDTH=\"55%\">$argList[$argIdx++]</TD>\n"; 	# Description
						$histList = "$histList</TR>\n";
					}
					$VAR{'PROBLEM_ID_VAR'} = &Link( $problemID, "$SCRIPT_PATH/web_access.cgi?FUNC=PROB_DETAIL&PROBLEM_ID=$problemID" );
					$VAR{'PROB_HIST_LIST_VAR'} = $histList;
					&SetAutoLogoutTag();
                    &set_prob_hist_strings();
					&EchoFile($HTML_PATH, "prob_hist.htm");
				}
				elsif ($argList[0] == 0) {
                    # There are no problem history records for this problem.
					$VAR{'INFORMATION_VAR'} = &InfoStyle( $NO_REQUEST_HISTORY, "RED" );
					&ProblemDetail();
				}
                else {
                    &AppServerError( 'GetProblemHistory', $argList[0] );
                }
				$done = 1;
			}
			else {
				$try++;
			}
	    } continue {
			$try++;
		}

		if($done eq 0) {
            &DaemonError( 'GetProblemHistory', $errCode );
		}
	}
	else {
        &LogItem( "! GetProblemHistory: Problem ID unknown" );
		&Error(); # Unexpected Error
	}
    &LogItem("<-- GetProblemHistory()");
}

###############################################################################
# Procedure:   GetWorkHistory
#
# FUNC:        VIEW_WORK_HISTORY
#
# Description: Display work history records for a given problem ID.
#
# Parameters:  (n/a)
#
# Return vals: (n/a)
###############################################################################
sub GetWorkHistory {
    &LogItem("--> GetWorkHistory()");
	&ValidateUser();

	if ($FORM{'PROBLEM_ID'}) {
		$problemID = $FORM{'PROBLEM_ID'};

		$try = 0;
		$done = 0;
		while(($try < $MAX_RETRIES)&&($done eq 0)) {
		    if(!&CreateSocket($DAEMON, $PORT)) {
				next;
			}
            &AddArg($KML_FILE, $STRING);
            &AddArg('WA_GetWorkHistory', $STRING);
			&AddArg($problemID, $STRING);
            &SendMsg($CGI_TO_DAEMON, $CD_APPSERVER_FUNCTION_NO_CALLER_CMD);
			($cmd, $subCmd, $errCode) = &RecvMsg();
			&CloseSocket();
			if(($cmd == $DAEMON_TO_CGI)&&($subCmd == $DC_APPSERVER_FUNCTION_RSP)&&($errCode == 0)) {
				$argIdx = 1;
				$histList = "";
				if($argList[0] > 0) {
					for($i=0; $i < $argList[0]; $i++) {
						$histList = "$histList<TR>\n";
						$date = &FormatDateStr($argList[$argIdx++]);
                        $time = &FormatTimeStr($argList[$argIdx++]);
						$histList = "$histList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"15%\">$date</TD>\n";  				# Begin Date
						$histList = "$histList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"15%\">$time</TD>\n";                # Begin Time
						$date = &FormatDateStr($argList[$argIdx++]);
                        $time = &FormatTimeStr($argList[$argIdx++]);
	                    $histList = "$histList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"15%\">$date</TD>\n";  				# End Date
	                    $histList = "$histList<TD ALIGN=CENTER VALIGN=TOP WIDTH=\"15%\">$time</TD>\n";                # End Time
						$histList = "$histList<TD ALIGN=LEFT VALIGN=TOP WIDTH=\"40%\">$argList[$argIdx++]</TD>\n"; 	# Description
						$histList = "$histList</TR>\n";
					}
					$VAR{'PROBLEM_ID_VAR'} = &Link( $problemID, "$SCRIPT_PATH/web_access.cgi?FUNC=PROB_DETAIL&PROBLEM_ID=$problemID" );
					$VAR{'WORK_HIST_LIST_VAR'} = $histList;
					&SetAutoLogoutTag();
                    &set_work_hist_strings();
					&EchoFile($HTML_PATH, "work_hist.htm");
				}
				elsif ($argList[0] == 0) {
                    # There are no work history records for this problem.
					$VAR{'INFORMATION_VAR'} = &InfoStyle( $NO_WORK_HISTORY, "RED" );
					&ProblemDetail();
				}
                else {
                    &AppServerError( 'GetWorkHistory', $argList[0] );
                }
				$done = 1;
			}
			else {
				$try++;
			}
	    } continue {
			$try++;
		}

		if($done eq 0) {
			&DaemonError( 'GetWorkHistory', $errCode );
		}
	}
	else {
        &LogItem( "! GetWorkHistory: Problem ID unknown" );
		&Error(); # Unexpected Error
	}
    &LogItem("<-- GetWorkHistory()");
}

# Needed for reuire statement in Main

1;
